##############################################################################
#
# Copyright (c) 2008 Agendaless Consulting and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################

from setuptools import setup

setup()



import sys
import os
import tempfile
import shutil
from io import StringIO

from docutils import nodes
from docutils.parsers.rst import directives, roles
from sphinx import application
from sphinx.builders.latex import LaTeXBuilder
from sphinx.pycode import ModuleAnalyzer


rootdir = os.path.abspath(os.path.dirname(__file__) or '.')


class ListOutput(object):
    """
    File-like object that collects written text in a list.
    """
    def __init__(self, name):
        self.name = name
        self.content = []

    def reset(self):
        del self.content[:]

    def write(self, text):
        self.content.append(text)


class SphinxTestApp(application.Sphinx):
    """
    A subclass of :class:`Sphinx` that runs on the test root, with some
    better default values for the initialization parameters.
    """

    def __init__(
        self,
        buildername='html',
        testroot=None,
        srcdir=None,
        freshenv=False,
        confoverrides=None,
        status=None,
        warning=None,
        tags=None,
        docutilsconf=None,
    ):
        if testroot is None:
            defaultsrcdir = 'root'
            testroot = os.path.join(rootdir, 'root')
        else:
            defaultsrcdir = 'test-' + testroot
            testroot = os.path.join(rootdir, 'roots', ('test-' + testroot))

        self.__tempdir = os.path.abspath(tempfile.mkdtemp())


        if srcdir is None:
            srcdir = os.path.join(self.__tempdir, defaultsrcdir)
        else:
            srcdir = os.path.join(self.__tempdir, srcdir)

        if not os.path.exists(srcdir):
            shutil.copytree(testroot, srcdir, symlinks=False)

        if docutilsconf is not None:
            with open(os.path.join(srcdir, 'docutils.conf')) as f:
                f.write(docutilsconf)

        builddir = os.path.join(srcdir, '_build')
#        if confdir is None:
        confdir = srcdir
#        if outdir is None:
        outdir = os.path.join(builddir, buildername)
        if not os.path.exists(outdir):
            os.makedirs(outdir)

        doctreedir = os.path.join(builddir, 'doctrees')
        if not os.path.exists(doctreedir):
            os.makedirs(doctreedir)
        if confoverrides is None:
            confoverrides = {}
        if status is None:
            status = StringIO()
        if warning is None:
            warning = ListOutput('stderr')
#        if warningiserror is None:
        warningiserror = False

        self._saved_path = sys.path[:]
        self._saved_directives = directives._directives.copy()
        self._saved_roles = roles._roles.copy()

        self._saved_nodeclasses = set(
            v for v in dir(nodes.GenericNodeVisitor)
            if v.startswith('visit_')
        )

        try:
            application.Sphinx.__init__(
                self,
                srcdir,
                confdir,
                outdir,
                doctreedir,
                buildername,
                confoverrides,
                status,
                warning,
                freshenv,
                warningiserror,
                tags,
            )
        except:
            self.cleanup()
            raise

    def cleanup(self, doctrees=False):
        shutil.rmtree(self.__tempdir)
        ModuleAnalyzer.cache.clear()
        LaTeXBuilder.usepackages = []
        sys.path[:] = self._saved_path
        sys.modules.pop('autodoc_fodder', None)
        directives._directives = self._saved_directives
        roles._roles = self._saved_roles
        for method in dir(nodes.GenericNodeVisitor):
            if (
                method.startswith('visit_') and
                method not in self._saved_nodeclasses
            ):
                delattr(nodes.GenericNodeVisitor, 'visit_' + method[6:])
                delattr(nodes.GenericNodeVisitor, 'depart_' + method[6:])

    def __repr__(self):
        klass_name = self.__class__.__name__
        builder_name = self.builder.name
        return f'<{klass_name} buildername={buildername:r}>'



import unittest

from docutils.statemachine import ViewList
from sphinx.ext.autodoc import ALL
from zope import interface

from repoze.sphinx import autointerface
from .util import SphinxTestApp

class IPlumbusMaker(interface.Interface):

    grumbo = interface.Attribute("The dinglebop is fed through here")
    fleeb = interface.Attribute("The dinglebop is polished with this")

    def smoothTheDinglebop(schleem):
        """Smooth it out

        The schleem is then repurposed.
        """

class Options(dict):
    inherited_members = False
    undoc_members = False
    private_members = False
    special_members = False
    imported_members = False
    show_inheritance = False
    noindex = False
    no_index = False  # FBO Sphinx 8.x
    no_index_entry = False  # FBO Sphinx 9.x
    annotation = None
    synopsis = ''
    platform = ''
    deprecated = False
    members = ()
    member_order = 'alphabetic'
    exclude_members = ()

    def __init__(self):
        super(Options, self).__init__()
        self.exclude_members = set()
        self.members = []
        self.__dict__ = self

class Settings(object):

    tab_width = 4


class Document(object):

    def __init__(self, settings):
        self.settings = settings


class State(object):

    def __init__(self, document):
        self.document = document


class Directive(object):
    env = None
    genopt = None
    result = None
    record_dependencies = None

    def __init__(self):
        self._warnings = []
        self.filename_set = set()
        self.result = ViewList()
        self.record_dependencies = set()
        self.state = State(Document(Settings()))

    def warn(self, msg):
        self._warnings.append(msg)

class AutoInterfaceTests(unittest.TestCase):

    def setUp(self):
        app = self.app = SphinxTestApp()
        app.builder.env.temp_data['docname'] = 'dummy'

        autointerface.setup(app)

        opt = self.options = Options()
        d = self.directive = Directive()
        d.env = app.builder.env
        d.genopt = opt

    def tearDown(self):
        self.app.cleanup()
        self.app = None

    def assertResultContains(
        self,
        item,
        objtype='interface',
        name='tests.unit.test_autointerface.IPlumbusMaker',
        **kw,
    ):
        directive = self.directive
        inst = self.app.registry.documenters['interface'](directive, name)
        inst.generate(**kw)
        # print '\n'.join(directive.result)
        self.assertEqual([], directive._warnings)
        self.assertIn(item, directive.result)
        results = directive.result[:]
        del directive.result[:]
        return '\n'.join(results)

    def test_restricted_members(self):
        self.options.members = ['smoothTheDinglebop']
        all_results = self.assertResultContains(
            '   .. method:: smoothTheDinglebop(schleem)'
        )
        self.assertNotIn('grumbo', all_results)

    def test_all_members(self):
        self.options.members = ALL
        all_results = self.assertResultContains(
            '   .. method:: smoothTheDinglebop(schleem)'
        )
        self.assertIn('grumbo', all_results)
        self.assertIn('fleeb', all_results)
